! Chris Hulme-Smith, University of Cambridge, 10th July 2017.
      PROGRAM MOLEDRIVE
! program to determine the molar driving force for the diffusionless
! transformation of austenite to ferrite using MTData.
      IMPLICIT NONE
! prevent FORTRAN compiler form using implied variable types and to
! ensure only explicitly-declared variables are used.  This minimises
! the chances of mistakes caused by incorrect types (e.g. integer used
! instead of floating point, etc.).

!**********************************************************************!
! declare all variables to be used in the main part of the program:
      INTEGER I0,MTDATA_MODE,COMP_UNITS,NO_OF_ALLOYING_ELEMENTS,ERR_ID
      INTEGER I1,THE_STEPPED_ELEMENT
! 'mode' specifies how MTData will open.
      PARAMETER(MTDATA_MODE=0)
!----------------------------------------------------------------------!
      DOUBLE PRECISION THE_COMP(0:100),TOTAL_COMP,THE_TEMP,THE_PRESS
      DOUBLE PRECISION START_STEP,STEP_VALUE,STOP_STEP,THE_TOTAL
      DOUBLE PRECISION MOLAR_GIBBS_ENERGY(3)
!----------------------------------------------------------------------!
      CHARACTER(LEN=100) DIRUSRAP_PATH
      CHARACTER(LEN=18) MPIFILE,MPRFILE
      CHARACTER(LEN=*) MTDATA_LOAD_OPTION,AUSTENITE,FERRITE
      PARAMETER(MTDATA_LOAD_OPTION='STAGE_1=NEW')
      PARAMETER(AUSTENITE='FCC_A1              ')
      PARAMETER(FERRITE='BCC_A2              ')
! read and use the contents of said file.  it includes definitions of
! the descriptive variable names used in theis program and must be in
! the specified path.
      INCLUDE 'DIRUSRAP.FOR'
!**********************************************************************!
! read in the contents of the input file:
! read in the name of the .mpi file used in the calculations
      PRINT *, 'Please enter the name of the .mpi file:'
      READ(*,*) MPIFILE
! read in the name of the .mpr file used as a results file for the
! calculations
      PRINT *, 'Please enter the name of the .mpr file:'
      READ(*,*) MPRFILE
!**********************************************************************!
! launch MTData:
! set the variable that records whether errors have occurred ot zero
! (i.e. no errors encountered yet!).
      ERR_ID=0
! this command launches MTDATA with a defined 'mode'.  In practice,
! this is always set to zero.
      CALL INITIALISE_MTDATA(MTDATA_MODE)
! ensure that specific memory units which are dedicated in the program
! for use for MTData are not used by other variables.
      CALL MTDATA_RESERVE_UNIT(13)
      CALL MTDATA_RESERVE_UNIT(14)
      CALL MTDATA_RESERVE_UNIT(15)
! enable error messages for the top level of MTData.
      CALL SGUMEN(1)
! enable error messages for Multiphase commands in MTData.
      CALL SGUMEN(7)
! tell MTData to write error logs to memory unit 14 (which is not used
! elsewhere in the program).
      CALL LOG_ERROR(14)
! create a new MTData calculation environment with all variables set to
! their default values.
      CALL MTOPTN(MTDATA_MODE,MTDATA_LOAD_OPTION)
! open the .mpi file and specify the name of the .mpr file to be made.
! Any error will change the value of err_id
      CALL OPEN_MPI_FILE(MPIFILE,MPRFILE,ERR_ID)
! sert the print level of the output.  For this type of program, MINIMAL
! is appropriate and is equivalent to an in teger value in the command
! of -2 (specified in DIRUSRAP.FOR; can replace 'MINIMAL' in the
! SET_PRINT_LEVEL command options with -2).
      CALL SET_PRINT_LEVEL(MINIMAL)
!**********************************************************************!
! specify the units of composition for these calculations
      PRINT *, 'Please enter the units for the composition - '
      PRINT *, '1 = wt.%, 2 = at.%, 3 = mass frac., 4 = mole frac.:'
      READ(*,*) COMP_UNITS
! specify the number of alloying elements in the composition in the
! .mpi file (can set unused elements to 0.0in the input file).
!      PRINT *, 'Please specify the number of alloying elements:'
!      READ(*,*) NO_OF_ALLOYING_ELEMENTS
! read in the composition of each alloying element and find the total
! content of alloying element.  Any stepped element will be removed in
! the next lines of code and then added again at each step.
      TOTAL_COMP=0.0
      DO I0=2,INIT_NO_OF_COMPONENTS(),1
        PRINT *, 'Please enter the content of ',
     & INIT_COMPONENT_NAME(I0)
        READ(*,*) THE_COMP(I0-1)
        TOTAL_COMP = TOTAL_COMP + THE_COMP(I0-1)
      END DO
! identify the stepped element and specify the starting value, step and
! end value for the content of that element
      PRINT *, 'Please specify which element is to be stepped:'
      DO I0=2,INIT_NO_OF_COMPONENTS(),1
        PRINT 100, I0-1,INIT_COMPONENT_NAME(I0)
      END DO
      READ(*,*) THE_STEPPED_ELEMENT
      PRINT *, 'Please specify the content for the first step:'
      READ(*,*) START_STEP
      PRINT *, 'Please enter the content by which to step:'
      READ(*,*) STEP_VALUE
      PRINT *, 'Please enter the content at which to stop stepping:'
      READ(*,*) STOP_STEP
      PRINT *, 'Please specify the temperature in Celsius:'
      READ(*,*) THE_TEMP
      THE_TEMP = THE_TEMP + 2.7315D+02
      PRINT *, 'Please specify the pressure in Pa:'
      READ(*,*) THE_PRESS
! remove the contribution from that element (will be added again at
! each step)
      TOTAL_COMP = TOTAL_COMP - THE_COMP(THE_STEPPED_ELEMENT)
!**********************************************************************!
! set up the global calculation environment:
      CALL SET_PRESSURE(THE_PRESS)
      CALL SET_TEMPERATURE(THE_TEMP)
! set the system size to 100kg, 100mol, 1kg or 1mol, as appropriate from
! the composition unit.
      IF(COMP_UNITS == 1 .OR. COMP_UNITS == 2) THEN ! at.%.
        THE_TOTAL = 100.0
      ELSE IF(COMP_UNITS == 3 .OR. COMP_UNITS == 4) THEN ! fraction.
        THE_TOTAL = 1.0
      ELSE
! if neither composition unit is used, print an error message and stop
! the program.
        PRINT 101
        STOP
      END IF
! print header information for the results
      PRINT 102
      PRINT 103
      DO I0=2,INIT_NO_OF_COMPONENTS(),1
        PRINT 104, INIT_COMPONENT_NAME(I0),THE_COMP(I0-1)
      END DO
      IF(COMP_UNITS == 1) THEN
        PRINT 105, INIT_COMPONENT_NAME(THE_STEPPED_ELEMENT+1),
     & START_STEP,STOP_STEP,STEP_VALUE,THE_TEMP-2.7315D+02,THE_PRESS,
     & 'wt%.           '
      ELSE IF(COMP_UNITS == 2) THEN
        PRINT 105, INIT_COMPONENT_NAME(THE_STEPPED_ELEMENT+1),
     & START_STEP,STOP_STEP,STEP_VALUE,THE_TEMP-2.7315D+02,THE_PRESS,
     & 'at.%.          '
      ELSE IF(COMP_UNITS == 3) THEN
        PRINT 105, INIT_COMPONENT_NAME(THE_STEPPED_ELEMENT+1),
     & START_STEP,STOP_STEP,STEP_VALUE,THE_TEMP-2.7315D+02,THE_PRESS,
     & 'mass fraction. '
      ELSE IF(COMP_UNITS == 4) THEN
        PRINT 105, INIT_COMPONENT_NAME(THE_STEPPED_ELEMENT+1),
     & START_STEP,STOP_STEP,STEP_VALUE,THE_TEMP-2.7315D+02,THE_PRESS,
     & 'molar fraction.'
      ELSE
        PRINT 101
        STOP
      END IF
      PRINT 107
!----------------------------------------------------------------------!
 100  FORMAT(I2,1X,A2)
 101  FORMAT('The unit of composition was not recognised.  The ',
     & 'program exits safely. Please retry: 1 = wt%, 2 = at.%, ',
     & '3 = mass fraction and 4 = mole fraction.')
 102  FORMAT('Driving force for diffusionless transformations in steel')
 103  FORMAT('Base composition:')
 104  FORMAT(2X,A2,2X,F10.6)
 105  FORMAT('Stepping ',A2,' from ',F10.6,' to',F10.6,
     & ' with increments of ',F10.6,' at a temperature of ',F7.2,
     & ' C and under a pressure of ',D11.6,' Pa.  All driving forces ',
     & ' are expressed as J/mol and the composition unit is ',A15)
 106  FORMAT('All driving forces are in J/mol:')
 107  FORMAT('    Step   G(Austenite)   G(Ferrite)   Driving force')
 108  FORMAT(F10.6,1X,2(D12.6,2X),F10.2)
!**********************************************************************!
! perform the calculation for each composition step:
! set the content of the stepped element to the initial value.
      THE_COMP(THE_STEPPED_ELEMENT) = START_STEP
!**********************************************************************!
! keep performing calculations until the maximum content of the stepped
! element is matched or exceeded.
      DO WHILE(THE_COMP(THE_STEPPED_ELEMENT) < STOP_STEP)
! determine the content of iron in the alloy
        THE_COMP(0) = THE_TOTAL-TOTAL_COMP-THE_COMP(THE_STEPPED_ELEMENT)
! set the composition of all elements (dangerous to set any element to 
!'undefined', but the way that the program is written renders this
! unnecessary)
        IF(COMP_UNITS == 1 .OR. COMP_UNITS == 3) THEN ! masses.
          DO I0=1,INIT_NO_OF_COMPONENTS(),1
            CALL SET_INIT_COMPONENT_MASS(I0,THE_COMP(I0-1))
          END DO
        ELSE IF(COMP_UNITS == 2 .OR. COMP_UNITS == 4) THEN ! moles.
          DO I0=1,INIT_NO_OF_COMPONENTS(),1
            CALL SET_INIT_COMPONENT_AMOUNT(I0,THE_COMP(I0-1))
          END DO
        END IF
!----------------------------------------------------------------------!
! set austenite to be the only permitted phase
        DO I0=1,INIT_NO_OF_PHASES(),1
          IF(INIT_PHASE_NAME(I0) == AUSTENITE) THEN
            CALL SET_INIT_PHASE_CLASS(I0,NORMAL)
          ELSE
            CALL SET_INIT_PHASE_CLASS(I0,ABSENT)
          END IF
        END DO
! calculate the molar free energy of the system
        CALL COMPUTE_EQUILIBRIUM()
        MOLAR_GIBBS_ENERGY(1) = SYSTEM_GIBBS_ENERGY() / 
     & SUM_OF_COMPONENT_AMOUNT()
!----------------------------------------------------------------------!
! set ferrite to be the only permitted phase
        DO I0=1,INIT_NO_OF_PHASES(),1
          IF(INIT_PHASE_NAME(I0) == FERRITE) THEN
            CALL SET_INIT_PHASE_CLASS(I0,NORMAL)
          ELSE
            CALL SET_INIT_PHASE_CLASS(I0,ABSENT)
          END IF
        END DO
! calculate the molar free energy of the system
        CALL COMPUTE_EQUILIBRIUM()
        MOLAR_GIBBS_ENERGY(2) = SYSTEM_GIBBS_ENERGY() / 
     & SUM_OF_COMPONENT_AMOUNT()
!----------------------------------------------------------------------!
! calculate the driving force
        MOLAR_GIBBS_ENERGY(3) = MOLAR_GIBBS_ENERGY(2) - 
     & MOLAR_GIBBS_ENERGY(1)
!**********************************************************************!
        PRINT 108, THE_COMP(THE_STEPPED_ELEMENT),
     & (MOLAR_GIBBS_ENERGY(I0),I0=1,3)
! increment the content of the stepped element by STEP_VALUE.
        THE_COMP(THE_STEPPED_ELEMENT) = THE_COMP(THE_STEPPED_ELEMENT)
     & + STEP_VALUE
      END DO
!**********************************************************************!
      END PROGRAM MOLEDRIVE